--local mjm = mjrequire "common/mjm"
--local vec3 = mjm.vec3


local locale = mjrequire "common/locale"
local resource = mjrequire "common/resource"
--local order = mjrequire "common/order"
local snapGroup = mjrequire "common/snapGroup"
local skill = mjrequire "common/skill"
local plan = mjrequire "common/plan"
local tool = mjrequire "common/tool"
local action = mjrequire "common/action"
local actionSequence = mjrequire "common/actionSequence"

local constructable = mjrequire "common/constructable"

local buildable = {}

local bringOnlySequence = {
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringResources.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringTools.index,
    },
}


local bringAndMoveSequence = {
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringResources.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringTools.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.moveComponents.index,
    },
}

local clearObjectsSequence = {
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.clearObjects.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringResources.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringTools.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.moveComponents.index,
    },
}

local clearObjectsAndTerrainSequence = {
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.clearObjects.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.clearTerrain.index
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.clearObjects.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringResources.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringTools.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.moveComponents.index,
    },
}

local plantSequence = {
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringResources.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringTools.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.actionSequence.index,
        actionSequenceTypeIndex = actionSequence.types.dig.index,
        requiredToolIndex = tool.types.dig.index, --must be available at the site, so this must be after constructable.sequenceTypes.bringResources
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.moveComponents.index,
        subModelAddition = {
            modelName = "plantHole"
        },
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.actionSequence.index,
        actionSequenceTypeIndex = actionSequence.types.dig.index,
        requiredToolIndex = tool.types.dig.index, --must be available at the site, so this must be after constructable.sequenceTypes.bringResources
        disallowCompletionWithoutSkill = true,
        subModelAddition = {
            modelName = "plantHole"
        },
    },
}



local fillSequence = {
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.clearObjects.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.clearTerrain.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.clearObjects.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringResources.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringTools.index,
    },
   --[[ {
        constructableSequenceTypeIndex = constructable.sequenceTypes.actionSequence.index,
        actionSequenceTypeIndex = actionSequence.types.dig.index,
        requiredToolIndex = tool.types.dig.index, --must be available at the site, so this must be after constructable.sequenceTypes.bringResources
    },]]
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.moveComponents.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.actionSequence.index,
        actionSequenceTypeIndex = actionSequence.types.dig.index,
        requiredToolIndex = tool.types.dig.index, --must be available at the site, so this must be after constructable.sequenceTypes.bringResources
        disallowCompletionWithoutSkill = true,
    },
}


local clearAndBringSequence = {
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.clearObjects.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.clearTerrain.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringResources.index,
    },
    {
        constructableSequenceTypeIndex = constructable.sequenceTypes.bringTools.index,
    },
}


local pathSequence = fillSequence



function buildable:addBuildable(key, buildableInfo)
	return constructable:addConstructable(key, buildableInfo)
end


buildable:addBuildable("craftArea", {
    modelName = "craftArea",
    inProgressGameObjectTypeKey = "build_craftArea",
    finalGameObjectTypeKey = "craftArea",
    name = locale:get("buildable_craftArea"),
    plural = locale:get("buildable_craftArea_plural"),
    summary = locale:get("buildable_craftArea_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.basicBuilding.index,
    },

    allowYTranslation = true,
    noBuildUnderWater = true,
    
    requiredResources = {
        {
            type = resource.types.rock.index,
            count = 1,
            afterAction = {
                actionTypeIndex = action.types.patDown.index,
            }
        }
    },

    placeBuildObjectsInFinalLocationsOnDropOff = true,

    requiresSlopeCheck = true,

    buildSequence = clearAndBringSequence,

    maleSnapPoints = snapGroup.malePoints.onFloor2x2MaleSnapPoints,
    snapToWalkableHeight = true,
})

buildable:addBuildable("storageArea", {
    modelName = "storageArea",
    inProgressGameObjectTypeKey = "build_storageArea",
    finalGameObjectTypeKey = "storageArea",
    name = locale:get("buildable_storageArea"),
    plural = locale:get("buildable_storageArea_plural"),
    summary = locale:get("buildable_storageArea_summary"),
    classification = constructable.classifications.build.index,
    
    allowYTranslation = false,
    noBuildUnderWater = true,
    requiresSlopeCheck = true,

    skills = {
        required = skill.types.basicBuilding.index,
    },

    --buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.onFloor2x2MaleSnapPoints,
    snapToWalkableHeight = true,
})

buildable:addBuildable("campfire", {
    modelName = "campfire",
    inProgressGameObjectTypeKey = "build_campfire",
    finalGameObjectTypeKey = "campfire",
    name = locale:get("buildable_campfire"),
    plural = locale:get("buildable_campfire_plural"),
    summary = locale:get("buildable_campfire_summary"),
    buildCompletionPlanIndex = plan.types.light.index,
    classification = constructable.classifications.build.index,
    allowYTranslation = false,
    noBuildUnderWater = true,
    requiresSlopeCheck = true,
    allowBuildEvenWhenDark = true,
    
    skills = {
        required = skill.types.fireLighting.index,
    },

    buildSequence = clearObjectsAndTerrainSequence,

    requiredResources = {
        {
            type = resource.types.rock.index,
            count = 6,
            afterAction = {
                actionTypeIndex = action.types.patDown.index,
            }
        }
    }
})


buildable:addBuildable("brickKiln", {
    modelName = "brickKiln",
    inProgressGameObjectTypeKey = "build_brickKiln",
    finalGameObjectTypeKey = "brickKiln",
    name = locale:get("buildable_brickKiln"),
    plural = locale:get("buildable_brickKiln_plural"),
    summary = locale:get("buildable_brickKiln_summary"),
    buildCompletionPlanIndex = plan.types.light.index,
    classification = constructable.classifications.build.index,
    allowYTranslation = false,
    noBuildUnderWater = true,
    requiresSlopeCheck = true,
    allowBuildEvenWhenDark = true,
    
    skills = {
        required = skill.types.mudBrickBuilding.index,
    },

    buildSequence = clearObjectsAndTerrainSequence,

    requiredResources = {
        {
            type = resource.types.mudBrickDry.index,
            count = 6,
            afterAction = {
                actionTypeIndex = action.types.patDown.index,
            }
        }
    }
})


buildable:addBuildable("torch", {
    modelName = "torch",
    inProgressGameObjectTypeKey = "build_torch",
    finalGameObjectTypeKey = "torch",
    name = locale:get("buildable_torch"),
    plural = locale:get("buildable_torch_plural"),
    summary = locale:get("buildable_torch_summary"),
    buildCompletionPlanIndex = plan.types.light.index,
    classification = constructable.classifications.build.index,
    allowBuildEvenWhenDark = true,

    skills = {
        required = skill.types.fireLighting.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,
    noBuildUnderWater = true,

    buildSequence = bringAndMoveSequence,
   -- placeBuildObjectsInFinalLocationsOnDropOff = true,
    
    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 1,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 1,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})

buildable:addBuildable("hayBed", {
    modelName = "hayBed",
    inProgressGameObjectTypeKey = "build_hayBed",
    finalGameObjectTypeKey = "hayBed",
    name = locale:get("buildable_hayBed"),
    plural = locale:get("buildable_hayBed_plural"),
    summary = locale:get("buildable_hayBed_summary"),
    classification = constructable.classifications.build.index,
    allowBuildEvenWhenDark = true,
    
    skills = {
        required = skill.types.basicBuilding.index,
    },

    requiresSlopeCheck = true,
    allowYTranslation = false,
    noBuildUnderWater = true,

    buildSequence = clearObjectsSequence,

    requiredResources = {
        {
            type = resource.types.hay.index,
            count = 3,
            afterAction = {
                actionTypeIndex = action.types.patDown.index,
                duration = 5.0,
            }
        },
    }
})


buildable:addBuildable("woolskinBed", {
    modelName = "woolskinBed",
    inProgressGameObjectTypeKey = "build_woolskinBed",
    finalGameObjectTypeKey = "woolskinBed",
    name = locale:get("buildable_woolskinBed"),
    plural = locale:get("buildable_woolskinBed_plural"),
    summary = locale:get("buildable_woolskinBed_summary"),
    classification = constructable.classifications.build.index,
    allowBuildEvenWhenDark = true,
    
    skills = {
        required = skill.types.basicBuilding.index,
    },

    requiresSlopeCheck = true,
    allowYTranslation = false,
    noBuildUnderWater = true,

    buildSequence = clearObjectsSequence,

    requiredResources = {
        {
            type = resource.types.woolskin.index,
            count = 3,
            afterAction = {
                actionTypeIndex = action.types.patDown.index,
                duration = 5.0,
            }
        },
    }
})

buildable:addBuildable("thatchRoof", {
    modelName = "thatchRoof",
    inProgressGameObjectTypeKey = "build_thatchRoof",
    finalGameObjectTypeKey = "thatchRoof",
    name = locale:get("buildable_thatchRoof"),
    plural = locale:get("buildable_thatchRoof_plural"),
    summary = locale:get("buildable_thatchRoof_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.thatchBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    
    maleSnapPoints = snapGroup.malePoints.roofMaleSnapPoints,

    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 4,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 8,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})


buildable:addBuildable("thatchRoofLarge", {
    modelName = "thatchRoofLarge",
    inProgressGameObjectTypeKey = "build_thatchRoofLarge",
    finalGameObjectTypeKey = "thatchRoofLarge",
    name = locale:get("buildable_thatchRoofLarge"),
    plural = locale:get("buildable_thatchRoofLarge_plural"),
    summary = locale:get("buildable_thatchRoofLarge_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.thatchBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    
    maleSnapPoints = snapGroup.malePoints.roofLargeMaleSnapPoints,

    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 6,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 8,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})

buildable:addBuildable("thatchRoofLargeCorner", {
    modelName = "thatchRoofLargeCorner",
    inProgressGameObjectTypeKey = "build_thatchRoofLargeCorner",
    finalGameObjectTypeKey = "thatchRoofLargeCorner",
    name = locale:get("buildable_thatchRoofLargeCorner"),
    plural = locale:get("buildable_thatchRoofLargeCorner_plural"),
    summary = locale:get("buildable_thatchRoofLargeCorner_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.thatchBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    
    maleSnapPoints = snapGroup.malePoints.roofLargeCornerMaleSnapPoints,

    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 9,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 9,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})

buildable:addBuildable("thatchWall", {
    modelName = "thatchWall",
    inProgressGameObjectTypeKey = "build_thatchWall",
    finalGameObjectTypeKey = "thatchWall",
    name = locale:get("buildable_thatchWall"),
    plural = locale:get("buildable_thatchWall_plural"),
    summary = locale:get("buildable_thatchWall_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.thatchBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,

    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 5,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 6,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})

buildable:addBuildable("thatchWallDoor", {
    modelName = "thatchWallDoor",
    inProgressGameObjectTypeKey = "build_thatchWallDoor",
    finalGameObjectTypeKey = "thatchWallDoor",
    name = locale:get("buildable_thatchWallDoor"),
    plural = locale:get("buildable_thatchWallDoor_plural"),
    summary = locale:get("buildable_thatchWallDoor_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.thatchBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,
    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 6,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 6,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})


buildable:addBuildable("thatchWallLargeWindow", {
    modelName = "thatchWallLargeWindow",
    inProgressGameObjectTypeKey = "build_thatchWallLargeWindow",
    finalGameObjectTypeKey = "thatchWallLargeWindow",
    name = locale:get("buildable_thatchWallLargeWindow"),
    plural = locale:get("buildable_thatchWallLargeWindow_plural"),
    summary = locale:get("buildable_thatchWallLargeWindow_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.thatchBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,

    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 5,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 6,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})

buildable:addBuildable("thatchRoofEnd", {
    modelName = "thatchRoofEnd",
    inProgressGameObjectTypeKey = "build_thatchRoofEnd",
    finalGameObjectTypeKey = "thatchRoofEnd",
    name = locale:get("buildable_thatchRoofEnd"),
    plural = locale:get("buildable_thatchRoofEnd_plural"),
    summary = locale:get("buildable_thatchRoofEnd_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.thatchBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.roofEndWallMaleSnapPoints,

    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 2,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 3,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})

buildable:addBuildable("thatchWall4x1", {
    modelName = "thatchWall4x1",
    inProgressGameObjectTypeKey = "build_thatchWall4x1",
    finalGameObjectTypeKey = "thatchWall4x1",
    name = locale:get("buildable_thatchWall4x1"),
    plural = locale:get("buildable_thatchWall4x1_plural"),
    summary = locale:get("buildable_thatchWall4x1_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.thatchBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wall4x1MaleSnapPoints,

    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 3,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 3,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})


buildable:addBuildable("thatchWall2x2", {
    modelName = "thatchWall2x2",
    inProgressGameObjectTypeKey = "build_thatchWall2x2",
    finalGameObjectTypeKey = "thatchWall2x2",
    name = locale:get("buildable_thatchWall2x2"),
    plural = locale:get("buildable_thatchWall2x2_plural"),
    summary = locale:get("buildable_thatchWall2x2_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.thatchBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wall2x2MaleSnapPoints,

    requiredResources = {
        {
            type = resource.types.branch.index,
            count = 3,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
        {
            type = resource.types.hay.index,
            count = 3,
            afterAction = {
                actionTypeIndex = action.types.inspect.index,
                durationWithoutSkill = 10.0,
            }
        },
    }
})

buildable:addBuildable("splitLogFloor", {
    modelName = "splitLogFloor2x2",
    inProgressGameObjectTypeKey = "build_splitLogFloor",
    finalGameObjectTypeKey = "splitLogFloor",
    name = locale:get("buildable_splitLogFloor"),
    plural = locale:get("buildable_splitLogFloor_plural"),
    summary = locale:get("buildable_splitLogFloor_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    maleSnapPoints = snapGroup.malePoints.floor2x2MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("splitLogFloor4x4", {
    modelName = "splitLogFloor4x4",
    inProgressGameObjectTypeKey = "build_splitLogFloor4x4",
    finalGameObjectTypeKey = "splitLogFloor4x4",
    name = locale:get("buildable_splitLogFloor4x4"),
    plural = locale:get("buildable_splitLogFloor4x4_plural"),
    summary = locale:get("buildable_splitLogFloor4x4_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    maleSnapPoints = snapGroup.malePoints.floor4x4MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("splitLogWall", {
    modelName = "splitLogWall",
    inProgressGameObjectTypeKey = "build_splitLogWall",
    finalGameObjectTypeKey = "splitLogWall",
    name = locale:get("buildable_splitLogWall"),
    plural = locale:get("buildable_splitLogWall_plural"),
    summary = locale:get("buildable_splitLogWall_summary"),
    classification = constructable.classifications.build.index,
    countsAsSplitLogWallForTutorial = true,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("splitLogWall4x1", {
    modelName = "splitLogWall4x1",
    inProgressGameObjectTypeKey = "build_splitLogWall4x1",
    finalGameObjectTypeKey = "splitLogWall4x1",
    name = locale:get("buildable_splitLogWall4x1"),
    plural = locale:get("buildable_splitLogWall4x1_plural"),
    summary = locale:get("buildable_splitLogWall4x1_summary"),
    classification = constructable.classifications.build.index,
    countsAsSplitLogWallForTutorial = true,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wall4x1MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("splitLogWall2x2", {
    modelName = "splitLogWall2x2",
    inProgressGameObjectTypeKey = "build_splitLogWall2x2",
    finalGameObjectTypeKey = "splitLogWall2x2",
    name = locale:get("buildable_splitLogWall2x2"),
    plural = locale:get("buildable_splitLogWall2x2_plural"),
    summary = locale:get("buildable_splitLogWall2x2_summary"),
    classification = constructable.classifications.build.index,
    countsAsSplitLogWallForTutorial = true,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wall2x2MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("splitLogWallDoor", {
    modelName = "splitLogWallDoor",
    inProgressGameObjectTypeKey = "build_splitLogWallDoor",
    finalGameObjectTypeKey = "splitLogWallDoor",
    name = locale:get("buildable_splitLogWallDoor"),
    plural = locale:get("buildable_splitLogWallDoor_plural"),
    summary = locale:get("buildable_splitLogWallDoor_summary"),
    classification = constructable.classifications.build.index,
    countsAsSplitLogWallForTutorial = true,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,

    requiredResources = {

    }
})


buildable:addBuildable("splitLogWallLargeWindow", {
    modelName = "splitLogWallLargeWindow",
    inProgressGameObjectTypeKey = "build_splitLogWallLargeWindow",
    finalGameObjectTypeKey = "splitLogWallLargeWindow",
    name = locale:get("buildable_splitLogWallLargeWindow"),
    plural = locale:get("buildable_splitLogWallLargeWindow_plural"),
    summary = locale:get("buildable_splitLogWallLargeWindow_summary"),
    classification = constructable.classifications.build.index,
    countsAsSplitLogWallForTutorial = true,

    skills = {
        required = skill.types.woodBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,

    requiredResources = {

    }
})


buildable:addBuildable("splitLogRoofEnd", {
    modelName = "splitLogRoofEnd",
    inProgressGameObjectTypeKey = "build_splitLogRoofEnd",
    finalGameObjectTypeKey = "splitLogRoofEnd",
    name = locale:get("buildable_splitLogRoofEnd"),
    plural = locale:get("buildable_splitLogRoofEnd_plural"),
    summary = locale:get("buildable_splitLogRoofEnd_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.woodBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.roofEndWallMaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("splitLogBench", {
    modelName = "splitLogBench",
    inProgressGameObjectTypeKey = "build_splitLogBench",
    finalGameObjectTypeKey = "splitLogBench",
    name = locale:get("buildable_splitLogBench"),
    plural = locale:get("buildable_splitLogBench_plural"),
    summary = locale:get("buildable_splitLogBench_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },

    requiresSlopeCheck = true,
    allowYTranslation = true,
    noBuildUnderWater = true,

    buildSequence = clearObjectsAndTerrainSequence,

    requiredResources = {

    }
})


buildable:addBuildable("splitLogSteps", {
    modelName = "splitLogSteps",
    inProgressGameObjectTypeKey = "build_splitLogSteps",
    finalGameObjectTypeKey = "splitLogSteps",
    name = locale:get("buildable_splitLogSteps"),
    plural = locale:get("buildable_splitLogSteps_plural"),
    summary = locale:get("buildable_splitLogSteps_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    maleSnapPoints = snapGroup.malePoints.steps1p5MaleSnapPoints,

    requiredResources = {

    }
})


buildable:addBuildable("splitLogSteps2x2", {
    modelName = "splitLogSteps2x2",
    inProgressGameObjectTypeKey = "build_splitLogSteps2x2",
    finalGameObjectTypeKey = "splitLogSteps2x2",
    name = locale:get("buildable_splitLogSteps2x2"),
    plural = locale:get("buildable_splitLogSteps2x2_plural"),
    summary = locale:get("buildable_splitLogSteps2x2_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    maleSnapPoints = snapGroup.malePoints.steps2HalfMaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("splitLogRoof", {
    modelName = "splitLogRoof",
    inProgressGameObjectTypeKey = "build_splitLogRoof",
    finalGameObjectTypeKey = "splitLogRoof",
    name = locale:get("buildable_splitLogRoof"),
    plural = locale:get("buildable_splitLogRoof_plural"),
    summary = locale:get("buildable_splitLogRoof_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.woodBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    
    maleSnapPoints = snapGroup.malePoints.roofMaleSnapPoints,

    requiredResources = {

    }
})


buildable:addBuildable("mudBrickWall", {
    modelName = "mudBrickWall",
    inProgressGameObjectTypeKey = "build_mudBrickWall",
    finalGameObjectTypeKey = "mudBrickWall",
    name = locale:get("buildable_mudBrickWall"),
    plural = locale:get("buildable_mudBrickWall_plural"),
    summary = locale:get("buildable_mudBrickWall_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.mudBrickBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("mudBrickWallDoor", {
    modelName = "mudBrickWallDoor",
    inProgressGameObjectTypeKey = "build_mudBrickWallDoor",
    finalGameObjectTypeKey = "mudBrickWallDoor",
    name = locale:get("buildable_mudBrickWallDoor"),
    plural = locale:get("buildable_mudBrickWallDoor_plural"),
    summary = locale:get("buildable_mudBrickWallDoor_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.mudBrickBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,
    requiredResources = {

    }
})


buildable:addBuildable("mudBrickWallLargeWindow", {
    modelName = "mudBrickWallLargeWindow",
    inProgressGameObjectTypeKey = "build_mudBrickWallLargeWindow",
    finalGameObjectTypeKey = "mudBrickWallLargeWindow",
    name = locale:get("buildable_mudBrickWallLargeWindow"),
    plural = locale:get("buildable_mudBrickWallLargeWindow_plural"),
    summary = locale:get("buildable_mudBrickWallLargeWindow_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.mudBrickBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,
    requiredResources = {

    }
})



buildable:addBuildable("mudBrickWall4x1", {
    modelName = "mudBrickWall4x1",
    inProgressGameObjectTypeKey = "build_mudBrickWall4x1",
    finalGameObjectTypeKey = "mudBrickWall4x1",
    name = locale:get("buildable_mudBrickWall4x1"),
    plural = locale:get("buildable_mudBrickWall4x1_plural"),
    summary = locale:get("buildable_mudBrickWall4x1_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.mudBrickBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wall4x1MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("mudBrickWall2x2", {
    modelName = "mudBrickWall2x2",
    inProgressGameObjectTypeKey = "build_mudBrickWall2x2",
    finalGameObjectTypeKey = "mudBrickWall2x2",
    name = locale:get("buildable_mudBrickWall2x2"),
    plural = locale:get("buildable_mudBrickWall2x2_plural"),
    summary = locale:get("buildable_mudBrickWall2x2_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.mudBrickBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wall2x2MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("mudBrickColumn", {
    modelName = "mudBrickColumn",
    inProgressGameObjectTypeKey = "build_mudBrickColumn",
    finalGameObjectTypeKey = "mudBrickColumn",
    name = locale:get("buildable_mudBrickColumn"),
    plural = locale:get("buildable_mudBrickColumn_plural"),
    summary = locale:get("buildable_mudBrickColumn_summary"),
    classification = constructable.classifications.build.index,

    skills = {
        required = skill.types.mudBrickBuilding.index,
    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.verticalColumnMaleSnapPoints,

    requiredResources = {

    }
})


buildable:addBuildable("mudBrickFloor2x2", {
    modelName = "mudBrickFloor2x2",
    inProgressGameObjectTypeKey = "build_mudBrickFloor2x2",
    finalGameObjectTypeKey = "mudBrickFloor2x2",
    name = locale:get("buildable_mudBrickFloor2x2"),
    plural = locale:get("buildable_mudBrickFloor2x2_plural"),
    summary = locale:get("buildable_mudBrickFloor2x2_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.mudBrickBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    maleSnapPoints = snapGroup.malePoints.floor2x2MaleSnapPoints,

    requiredResources = {

    }
})


buildable:addBuildable("mudBrickFloor4x4", {
    modelName = "mudBrickFloor4x4",
    inProgressGameObjectTypeKey = "build_mudBrickFloor4x4",
    finalGameObjectTypeKey = "mudBrickFloor4x4",
    name = locale:get("buildable_mudBrickFloor4x4"),
    plural = locale:get("buildable_mudBrickFloor4x4_plural"),
    summary = locale:get("buildable_mudBrickFloor4x4_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {
        required = skill.types.mudBrickBuilding.index,
    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    maleSnapPoints = snapGroup.malePoints.floor4x4MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("brickWall", {
    modelName = "brickWall",
    inProgressGameObjectTypeKey = "build_brickWall",
    finalGameObjectTypeKey = "brickWall",
    name = locale:get("buildable_brickWall"),
    plural = locale:get("buildable_brickWall_plural"),
    summary = locale:get("buildable_brickWall_summary"),
    classification = constructable.classifications.build.index,

    skills = {

    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,

    requiredResources = {

    }
})


buildable:addBuildable("brickWallDoor", {
    modelName = "brickWallDoor",
    inProgressGameObjectTypeKey = "build_brickWallDoor",
    finalGameObjectTypeKey = "brickWallDoor",
    name = locale:get("buildable_brickWallDoor"),
    plural = locale:get("buildable_brickWallDoor_plural"),
    summary = locale:get("buildable_brickWallDoor_summary"),
    classification = constructable.classifications.build.index,

    skills = {

    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,
    requiredResources = {

    }
})



buildable:addBuildable("brickWallLargeWindow", {
    modelName = "brickWallLargeWindow",
    inProgressGameObjectTypeKey = "build_brickWallLargeWindow",
    finalGameObjectTypeKey = "brickWallLargeWindow",
    name = locale:get("buildable_brickWallLargeWindow"),
    plural = locale:get("buildable_brickWallLargeWindow_plural"),
    summary = locale:get("buildable_brickWallLargeWindow_summary"),
    classification = constructable.classifications.build.index,

    skills = {

    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wallMaleSnapPoints,
    requiredResources = {

    }
})



buildable:addBuildable("brickWall4x1", {
    modelName = "brickWall4x1",
    inProgressGameObjectTypeKey = "build_brickWall4x1",
    finalGameObjectTypeKey = "brickWall4x1",
    name = locale:get("buildable_brickWall4x1"),
    plural = locale:get("buildable_brickWall4x1_plural"),
    summary = locale:get("buildable_brickWall4x1_summary"),
    classification = constructable.classifications.build.index,

    skills = {

    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wall4x1MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("brickWall2x2", {
    modelName = "brickWall2x2",
    inProgressGameObjectTypeKey = "build_brickWall2x2",
    finalGameObjectTypeKey = "brickWall2x2",
    name = locale:get("buildable_brickWall2x2"),
    plural = locale:get("buildable_brickWall2x2_plural"),
    summary = locale:get("buildable_brickWall2x2_summary"),
    classification = constructable.classifications.build.index,

    skills = {

    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,

    maleSnapPoints = snapGroup.malePoints.wall2x2MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("tileFloor2x2", {
    modelName = "tileFloor2x2",
    inProgressGameObjectTypeKey = "build_tileFloor2x2",
    finalGameObjectTypeKey = "tileFloor2x2",
    name = locale:get("buildable_tileFloor2x2"),
    plural = locale:get("buildable_tileFloor2x2_plural"),
    summary = locale:get("buildable_tileFloor2x2_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {

    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    maleSnapPoints = snapGroup.malePoints.floor2x2MaleSnapPoints,

    requiredResources = {

    }
})


buildable:addBuildable("tileFloor4x4", {
    modelName = "tileFloor4x4",
    inProgressGameObjectTypeKey = "build_tileFloor4x4",
    finalGameObjectTypeKey = "tileFloor4x4",
    name = locale:get("buildable_tileFloor4x4"),
    plural = locale:get("buildable_tileFloor4x4_plural"),
    summary = locale:get("buildable_tileFloor4x4_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {

    },

    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    maleSnapPoints = snapGroup.malePoints.floor4x4MaleSnapPoints,

    requiredResources = {

    }
})

buildable:addBuildable("tileRoof", {
    modelName = "tileRoof",
    inProgressGameObjectTypeKey = "build_tileRoof",
    finalGameObjectTypeKey = "tileRoof",
    name = locale:get("buildable_tileRoof"),
    plural = locale:get("buildable_tileRoof_plural"),
    summary = locale:get("buildable_tileRoof_summary"),
    classification = constructable.classifications.build.index,
    
    skills = {

    },
    
    allowYTranslation = true,
    allowXZRotation = true,

    buildSequence = clearObjectsAndTerrainSequence,
    
    maleSnapPoints = snapGroup.malePoints.roofMaleSnapPoints,

    requiredResources = {

    }
})

function buildable:addInfoForPlaceableResource(baseResourceType, defaultModelName, defaultGameObjectTypeKey)
    local buildableKey = "place_" .. baseResourceType.key
    if not constructable.types[buildableKey] then
        buildable:addBuildable(buildableKey,{
            name = locale:get("misc_decorate_with", {name = baseResourceType.name}),
            plural = baseResourceType.plural,
            modelName = defaultModelName,
            inProgressGameObjectTypeKey = "place_" .. defaultGameObjectTypeKey,
            finalGameObjectTypeKey = "placed_" .. defaultGameObjectTypeKey,
            summary = locale:get("misc_place_object_summary"),
            classification = constructable.classifications.place.index,
    
            canAttachToAnyObjectWithoutTestingForCollisions = true,
            isPlaceType = true,
            disallowsDecorationPlacing = baseResourceType.disallowsDecorationPlacing,
            allowXZRotation = true,
            allowYTranslation = true,
            
            buildSequence = bringOnlySequence,
            placeBuildObjectsInFinalLocationsOnDropOff = true,
            placeResourceTypeIndex = baseResourceType.index,
            
            skills = {
                required = skill.types.basicBuilding.index,
            },
    
            requiredResources = {
                {
                    type = baseResourceType.index,
                    --objectTypeIndex = baseObjectType.index,
                    count = 1,
                },
            },

            maleSnapPoints = baseResourceType.placeBuildableMaleSnapPoints,
        })
    end
end

function buildable:addInfoForPlantableObject(baseObjectType, summary, seedResourceTypeIndex, mediumTypes, inProgressModelName)

    
    --[[buildSequence = {
        {
            buildSequenceTypeIndex = "clearObjects",
        },
        {
            buildSequenceTypeIndex = "actionSequence",
            actionSequenceTypeIndex = actionSequence.types.dig.index,
            duration = 10.0,
            requireToolIndex = tool.types.dig.index,
        },
        {
            buildSequenceTypeIndex = "moveComponents",
        },
    },]]

    local buildableKey = "plant_" .. baseObjectType.key
    if not constructable.types[buildableKey] then
        buildable:addBuildable(buildableKey,{
            name = baseObjectType.name,
            plural = baseObjectType.plural,
            modelName = baseObjectType.modelName,
            iconGameObjectType = baseObjectType.index,--resourceType.displayGameObjectTypeIndex,
            --placeholderOverrideModelName = "craftSimple",
            inProgressBuildModel = inProgressModelName,
            inProgressGameObjectTypeKey = "plant_" .. baseObjectType.key,
            finalGameObjectTypeKey = "sapling_" .. baseObjectType.key,
            summary = summary,
            randomInitialYRotation = true,
            planTypeIndex = plan.types.plant.index,
            classification = constructable.classifications.plant.index,

            disallowsLimitedAbilitySapiens = true,
            
            buildSequence = plantSequence,
            
            requiredMediumTypes = mediumTypes,

            skills = {
                required = skill.types.planting.index,
            },
    
            requiredResources = {
                {
                    type = seedResourceTypeIndex,
                    --objectTypeIndex = baseObjectType.index,
                    count = 1,
                    afterAction = {
                        actionTypeIndex = action.types.patDown.index,
                        duration = 5.0,
                        durationWithoutSkill = 15.0,
                    }
                },
            },
            requiredTools = {
                tool.types.dig.index,
            },
        })
    end

    return constructable.types[buildableKey].index
end

function buildable:addInfoForFillResource(resourceTypeIndex, requiredResourceCount)
    local resourceType = resource.types[resourceTypeIndex]
    local buildableKey = "fill_" .. resourceType.key
    local requiredSkillTypeIndex = skill.types.digging.index
    local requiredToolTypeIndex = tool.types.dig.index

    if not constructable.types[buildableKey] then
        buildable:addBuildable(buildableKey,{
            name = resourceType.plural,
            plural = resourceType.plural,
            --modelName = objectType.modelName,
            inProgressBuildModel = "craftSimple",
           -- placeholderOverrideModelName = "craftSimple",
            summary = locale:get("fill_summary", { resourceTypeNamePlural = resourceType.plural, requiredResourceCount = requiredResourceCount}),
            planTypeIndex = plan.types.fill.index,
            classification = constructable.classifications.fill.index,
            
            iconGameObjectType = resourceType.displayGameObjectTypeIndex,

            disallowsLimitedAbilitySapiens = true,
            omitFromDiscoveryUI = true,
            
            buildSequence = fillSequence,
            

            skills = {
                required = requiredSkillTypeIndex,
            },

            requiredResources = {
                {
                    type = resourceTypeIndex,
                    count = requiredResourceCount,
                },
            },
            requiredTools = {
                requiredToolTypeIndex,
            },
        })
    end
end

function buildable:addInfoForPathObject(baseObjectKey, 
    name, 
    plural, 
    summary,
    modelName, 
    defaultSubModelName, 
    mainModelNameByObjectTypeIndexFunction, 
    subModelNameByObjectTypeIndexFunction, 
    requiredResourceTypeIndex, 
    requiredResourceCount,
    requiredSkillTypeIndex)

    local buildableKey = baseObjectKey
    if not constructable.types[buildableKey] then
        buildable:addBuildable(buildableKey,{
            name = name,
            plural = plural,
            modelName = modelName,
            defaultSubModelName = defaultSubModelName,
            subModelNameByObjectTypeIndexFunction = subModelNameByObjectTypeIndexFunction,
            mainModelNameByObjectTypeIndexFunction = mainModelNameByObjectTypeIndexFunction,
            inProgressGameObjectTypeKey = "build_" .. baseObjectKey,
            finalGameObjectTypeKey = baseObjectKey,
            summary = summary,
            classification = constructable.classifications.path.index,
            randomInitialYRotation = true,
            planTypeIndex = plan.types.buildPath.index,
            offsetModelToFitTerrain = true,
            
            buildSequence = pathSequence,

            disallowsLimitedAbilitySapiens = true,
            
            isPathType = true,
            --offsetToWalkableHeightTestRadius = 1.0,
            
            skills = {
                required = requiredSkillTypeIndex,--skill.types.digging.index,
            },
    
    
            requiredResources = {
                {
                    type = requiredResourceTypeIndex,
                    --objectTypeIndex = baseObjectType.index,
                    count = requiredResourceCount,
                    afterAction = {
                        actionTypeIndex = action.types.patDown.index,
                    }
                },
            },
            requiredTools = {
                tool.types.dig.index,
            },
        })
    end
end



return buildable